package owg.lgm;

import owg.lgm.util.AutoDestructibleImpl;
import owg.lgm.util.FILE;
import java.io.IOException;
import java.util.HashMap;

import com.jogamp.opengl.GL2ES1;

import owg.lgm.gl.Tex2D;
import owg.lgm.gl.TexSampler;
import owg.lgm.gl.Wrap;
import owg.util.fileextensions.FileExtensions;
import owg.lgm.plaf.PNGIO;

public class Background extends AutoDestructibleImpl {
	public static final HashMap<String, Background> registry = new HashMap<>();

	public static void load(GL2ES1 gl, FILE current) throws IOException
	{
		String ext = FileExtensions.getFileExtension(current.getPath());
		if(!"png".equals(ext))
			return;
		//System.out.println("Loading Background: "+current);
		IImage image = PNGIO.load(current);
		Background bg = new Background(gl, FileExtensions.stripFileExtension(image.name), image, false);
		if(registry.put(bg.name, bg) != null)
			System.err.println("Background name collision: "+bg.name);
	}
	
	public static Background add(GL2ES1 gl, FILE file, boolean smooth)
	{
		try {
			IImage image = PNGIO.load(file);
			Background bg = new Background(gl, null, image, smooth);
			return bg;
		} catch(IOException err) {
			err.printStackTrace();
			return null;
		}
	}
	
	public static Background get(String name)
	{
		Background r = registry.get(name);
		if(r == null)
			throw new ResourceException("No such background: "+name);
		return r;
	}
	
	public final String name;
	public final int width, height;
	public final IImage image;

	protected TexSampler sampler;
	
	public final Tex2D tex;
	
	protected Background(GL2ES1 gl, String name, IImage data, boolean smooth)
	{
		this.name = name;
		this.image = data;
		this.width = data.width;
		this.height = data.height;
		
		sampler = smooth?TexSampler.LINEAR_CLAMP:TexSampler.NEAREST_CLAMP;
		this.tex = new Tex2D(gl, image, false, sampler);
		tex.setName(name);
		autoDestroy(tex);
	}
	
	@Override
	public String toString()
	{
		return name;
	}

	public void setTextureRepeat(boolean repeat)
	{
		sampler = new TexSampler(
				sampler.minFilter,
				sampler.magFilter,
				repeat ? Wrap.REPEAT : Wrap.CLAMP_TO_EDGE, 
				repeat ? Wrap.REPEAT : Wrap.CLAMP_TO_EDGE
		);
		updateSampler();
	}
	
	public void setSampler(TexSampler sampler)
	{
		this.sampler = sampler;
		updateSampler();
	}

	protected void updateSampler()
	{
		tex.setSampler(sampler);
	}
}
