package owg.lgm.gl;

import static owg.lgm.plaf.Engine.EN;

import com.jogamp.opengl.GL;

public enum BlendMode {
	/**Replaces the destination with the source. This is identical to not using blending at all.*/
	REPLACE(GL.GL_ONE, GL.GL_ZERO),
	/**Blends the source onto the destination. The source color is assumed to not be premultiplied.*/
	NORMAL(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA),
	/**Blends the source onto the destination. The source color is assumed to be premultiplied.*/
	NORMAL_PRE(GL.GL_ONE, GL.GL_ONE_MINUS_SRC_ALPHA),
	/**Adds the source to the destination. The source color will be multiplied by the source alpha.*/
	ADD(GL.GL_SRC_ALPHA, GL.GL_ONE),
	/**Adds the source to the destination. The source color is assumed to be premultiplied, 
	 * so it is not multiplied by the source alpha.*/
	ADD_PRE(GL.GL_ONE, GL.GL_ONE),
	/**Inverts the destination color around gray, by a factor defined by the source color. 
	 * A value of 1 indicates complete inversion for the channel, and 0 will leave it unaffected. 
	 * The alpha channel is not used.*/
	INVERT(GL.GL_ONE_MINUS_DST_COLOR, GL.GL_ONE_MINUS_SRC_COLOR),
	/**Multiplies the destination color by the source color. Assuming the source color is premultiplied, 
	 * the alpha channel indicates how much each pixel should be affected.*/
	MULTIPLY(GL.GL_DST_COLOR, GL.GL_ONE_MINUS_SRC_ALPHA),
	/**Simulates subtractve blending by multiplying the destination color by the inverse of the source color.
	 * Assuming the source color is premultiplied, the alpha channel indicates how much each pixel should be affected.*/
	SUBTRACT(GL.GL_ZERO, GL.GL_ONE_MINUS_SRC_COLOR);
	
	public final int src, dst;
	
	BlendMode(int src, int dst) {
		this.src = src;
		this.dst = dst;
	}
	
	public void set()
	{
		EN.gl.glBlendFunc(src, dst);
	}
}
