package owg.lgm.gl;

import java.nio.Buffer;

import com.jogamp.opengl.GL2ES1;
import com.jogamp.opengl.util.GLBuffers;

public abstract class BufferObject extends GLResource
{
	public final int target;
	public final BufferUsage usage;
	
	protected transient GL2ES1 gl;
	protected transient int handle;
	
	protected int eOffset = -1, eSize = -1;
	
	public BufferObject(int target, int eOffset, int eSize, BufferUsage usage)
	{
		super();
		this.target = target;
		this.eOffset = eOffset;
		this.eSize = eSize;
		this.usage = usage;
	}

	@Override
	public int getInitOrder() {
		return 0;
	}
	
	@Override
	public void init(GL2ES1 gl)
	{
		this.gl = gl;
		{
			final int[] r = new int[1];
			gl.glGenBuffers(1, r, 0);
			handle = r[0];
		}
		
		if(eSize == 0)
			new Throwable("Warning: Empty buffer").printStackTrace();
		replace();
	}
	
	protected void replace()
	{
		gl.glBindBuffer(target, handle);
		Buffer data = getData();
		data.mark();
		data.position(eOffset);
		gl.glBufferData(target, eSize*GLBuffers.sizeOfBufferElem(data), data, usage.value);
		data.reset();
		gl.glBindBuffer(target, 0);
	}
	
	public void update()
	{
		gl.glBindBuffer(target, handle);
		Buffer data = getData();
		data.mark();
		data.position(eOffset);
		gl.glBufferSubData(target, 0, eSize*GLBuffers.sizeOfBufferElem(data), data);
		data.reset();
		gl.glBindBuffer(target, 0);
	}
	
	public void update(int offset, int len)
	{
		gl.glBindBuffer(target, handle);
		Buffer data = getData();
		int mark = data.position();
		int bpp = GLBuffers.sizeOfBufferElem(data);
		data.position(eOffset+offset);
		gl.glBufferSubData(target, offset*bpp, len*bpp, data);
		data.position(mark);
		gl.glBindBuffer(target, 0);
	}
	
	@Override
	public void uninit()
	{
		gl.glDeleteBuffers(1, new int[] {handle}, 0);
		handle = 0;
	}
	
	protected abstract Buffer getData();
	
	public GL2ES1 getGL()
	{
		return gl;
	}
	
	public int getHandle()
	{
		return handle;
	}
}
