package owg.lgm.gl;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import com.jogamp.opengl.GL2ES1;

import owg.lgm.gl.DrawCommand.DrawArrays;
import owg.lgm.util.AutoDestructibleImpl;
import owg.util.autoname.AutoName;

public class PrimitiveBuffer extends AutoDestructibleImpl
{
	static {
		AutoName.addLibraryClass(PrimitiveBuffer.class);
	}
	
	public static class VertexComponent
	{
		public final VertexAttribute attrib;
		public final int elementCount;
		public VertexComponent(VertexAttribute attrib, int elementCount)
		{
			this.attrib = attrib;
			this.elementCount = elementCount;
		}
	}
	
	public final int bufferSizeVerts;
	public final int componentCount;
	protected FloatBuffer verts;

	protected boolean modify;
	protected boolean replace;
	protected final VertexBufferObjectF vbo;
	protected final DrawArrays drawCommand;
	protected final Mesh mesh;
	
	public PrimitiveBuffer(GL2ES1 gl, int bufferSizeVerts, BufferUsage usage, VertexComponent... components)
	{
		this.bufferSizeVerts = bufferSizeVerts;
		int cc = 0;
		for(int i = 0; i<components.length; i++)
			cc += components[i].elementCount;
		this.componentCount = cc;
		verts = ByteBuffer.allocateDirect(4*bufferSizeVerts*componentCount).order(ByteOrder.LITTLE_ENDIAN).asFloatBuffer();
		
		vbo = new VertexBufferObjectF(gl, verts, usage);
		autoDestroy(vbo);
		modify = false;
		replace = false;
		
		drawCommand = DrawCommand.drawArrays(PrimitiveType.POINTS, 0, 0);
		AttribArray[] attribs = new AttribArray[components.length];
		cc = 0;
		for(int i = 0; i<components.length; i++)
		{
			attribs[i] = new AttribArray(vbo, components[i].attrib, components[i].elementCount, DataType.FLOAT, 4*componentCount, 4*cc);
			cc += components[i].elementCount;
		}
		
		mesh = new Mesh(drawCommand, attribs);
		autoDestroy(mesh);
	}
	
	public GL2ES1 getGL()
	{
		return vbo.getGL();
	}

	protected FloatBuffer buffer()
	{
		if(!verts.hasRemaining())
		{
			FloatBuffer r = ByteBuffer.allocateDirect(4*(verts.capacity() + bufferSizeVerts*componentCount)).order(ByteOrder.LITTLE_ENDIAN).asFloatBuffer();
			verts.flip();
			r.put(verts);
			verts = r;
			replace = true;
		}
		modify = true;
		return verts;
	}
	
	public void begin(PrimitiveType kind)
	{
		drawCommand.mode = kind;
		verts.clear();
		modify = true;
	}
	
	public void render()
	{
		if(replace)
		{
			int mark = verts.position();
			verts.rewind();
			vbo.setData(verts);
			verts.position(mark);
		}
		else if(modify)
		{
			vbo.update(0, verts.position());
		}
		replace = false;
		modify = false;
		
		drawCommand.count = verts.position()/componentCount;
		mesh.render(vbo.getGL());
	}
}

