package owg.lgm.plaf;

import static owg.lgm.GMLib.color_get_blue;
import static owg.lgm.GMLib.color_get_green;
import static owg.lgm.GMLib.color_get_red;

import java.io.File;
import java.io.IOException;
import java.nio.FloatBuffer;

import com.jogamp.common.util.IOUtil.ClassResources;
import com.jogamp.nativewindow.WindowClosingProtocol.WindowClosingMode;
import com.jogamp.nativewindow.util.RectangleImmutable;
import com.jogamp.newt.Display;
import com.jogamp.newt.NewtFactory;
import com.jogamp.newt.Screen;
import com.jogamp.newt.opengl.GLWindow;
import com.jogamp.opengl.DebugGL2;
import com.jogamp.opengl.GL2ES1;
import com.jogamp.opengl.GLAutoDrawable;
import com.jogamp.opengl.GLCapabilities;
import com.jogamp.opengl.GLEventListener;
import com.jogamp.opengl.GLProfile;
import com.jogamp.opengl.glu.GLU;
import com.jogamp.opengl.glu.gl2.GLUgl2;
import com.jogamp.opengl.util.GLBuffers;

import owg.lgm.Activity;
import owg.lgm.Fonts;
import owg.lgm.GMLib;
import owg.lgm.GameConfig;
import owg.lgm.ResourceException;
import owg.lgm.gl.AttribArray;
import owg.lgm.gl.BufferUsage;
import owg.lgm.gl.DataType;
import owg.lgm.gl.DrawCommand;
import owg.lgm.gl.GLResource;
import owg.lgm.gl.Mesh;
import owg.lgm.gl.PrimitiveType;
import owg.lgm.gl.VertexAttribute;
import owg.lgm.gl.VertexBufferObjectF;
import owg.lgm.in.InputState;
import owg.lgm.in.VK;
import owg.lgm.loader.LoadingScreen;
import owg.lgm.room.ActivityChangeException;
import owg.lgm.room.ActivityChangeType;
import owg.lgm.room.Room;
import owg.lgm.room.RoomData;
import owg.lgm.util.FramebufferManager;
import owg.lgm.util.FramebufferManagerDummy;
import owg.lgm.util.FramebufferManagerImpl;
import owg.lgm.util.Reflect;
import owg.util.applicationdirectory.ApplicationDirectoryFinder;
import owg.util.autoname.AutoName;
import owg.util.printstreamlogger.PrintStreamLogger;

public class Engine implements GLEventListener
{
	public static final boolean IS_DESKTOP = true;
	
	public static Engine EN;
	
	protected enum Status
	{
		INITIAL,
		RUNNING,
		ENDED;
	}
	
	public final File dir = ApplicationDirectoryFinder.findApplicationDirectory(Engine.class);

	public final InputHandler ih;
	public InputState in;
	public final GameConfig gameConfig;
	public FramebufferManager fbm;

	public final Display display;
	public final Screen screen;
	public final GLWindow window;
	public final UltimateAnimator animator;
	
	protected volatile boolean windowSizeSet = false;
	protected volatile boolean windowDestroyed = false;
	protected boolean vSync = false;
	protected Status status = Status.INITIAL;
	public Activity act;
	
	public Mesh nwRect;
	public long time;
	
	public Engine(GameConfig gameConfig, String[] args, Runnable startAction) throws IOException
	{
    	for(int i = 0; i<args.length; i++)
    	{
    		int st = 0;
    		while(st<args[i].length() && args[i].charAt(st) == '-')
    			st++;
    		int sep = args[i].indexOf('=', st);
    		if(sep != -1)
    			System.setProperty(args[i].substring(st, sep), args[i].substring(sep+1));
    	}
		
		PrintStreamLogger.replaceOut();
		System.out.println("Game Directory: "+dir);
		AutoName.enabled = true;
		EN = this;
		this.gameConfig = gameConfig;

		if(gameConfig.icons != null)
			NewtFactory.setWindowIcons(new ClassResources(gameConfig.icons, getClass().getClassLoader(), gameConfig.getClass()));
		
    	ih = new InputHandler();
    	LoadingScreen loader = new LoadingScreen(startAction);
		act = loader;
		display = NewtFactory.createDisplay(null);
		screen = NewtFactory.createScreen(display, 0);
		GLProfile profile = GLProfile.get(GLProfile.GL2);
		GLCapabilities caps = new GLCapabilities(profile);
		caps.setDepthBits(24);
		caps.setStencilBits(8);
		caps.setRedBits(8);
		caps.setGreenBits(8);
		caps.setBlueBits(8);
		window = GLWindow.create(screen, caps);
		window.setTitle(gameConfig.name);
		window.setUndecorated(gameConfig.borderless);
		window.setSurfaceSize(loader.getWidth(), loader.getHeight());
		
		window.setDefaultCloseOperation(WindowClosingMode.DO_NOTHING_ON_CLOSE);
    	window.addGLEventListener(this);
    	
    	ih.init(window);
		
		animator = new UltimateAnimator(30, false);
    	
		window.setVisible(false, true);
		animator.start();//Hack to prevent avian shutdown
	}
	
	public Engine(GameConfig gameConfig, String[] args) throws IOException
	{
		this(gameConfig, args, () -> {
			RoomData.mangleCreationCodes();
			EN.loadingComplete();
			throw new ActivityChangeException(ActivityChangeType.GAME_START, RoomData.list.get(0));
		});
	}

	protected boolean fullscreen = false;
	
	public void setFullscreen(final boolean b)
	{
		fullscreen = b;
		animator.invokeOnAnimator(() -> {
			if(window.isFullscreen() != b)
			{
				window.setFullscreen(b);
				ih.clear();
				if(!b)
					validateWindowSize();
			}
		});
	}
	
	public void loadingComplete()
	{
		fullscreen = gameConfig.fullscreen;
		animator.invokeOnAnimator(new Runnable() {
			@Override
			public void run()
			{
				if(gameConfig.fullscreen)
					window.setFullscreen(true);
				else 
					validateWindowSize();
				window.setResizable(gameConfig.resizeable);
			}
		});
	}

	protected void validateWindowSize()
	{
		if(!windowSizeSet)
		{
			RectangleImmutable vp = window.getMainMonitor().getViewport();
			window.setPosition(vp.getX()+(vp.getWidth()-EN.gameConfig.width)/2, vp.getY()+(vp.getHeight()-EN.gameConfig.height)/2);
			window.setSurfaceSize(EN.gameConfig.width, EN.gameConfig.height);
			windowSizeSet = true;
		}
	}

	public boolean isFullscreen()
	{
		return fullscreen;
	}
	
	public GL2ES1 gl;
	public GLU glu;
	
	@Override
	public void init(GLAutoDrawable drawable)
	{
		if(status == Status.ENDED)
			return;
		
		XKBFix.apply(display, window);
		gl = new DebugGL2(drawable.getGL().getGL2());
		glu = new GLUgl2();
		GLResource.initAll(gl);
		
		String vSync = System.getProperty("gl.vsync");
		if("true".equals(vSync))
			setVSync(true);
		else if("false".equals(vSync))
			setVSync(false);
		
		if(status == Status.INITIAL)
		{
			FloatBuffer nwVerts = GLBuffers.newDirectFloatBuffer(4*2);
			nwVerts.put(0).put(0).put(1).put(0).put(0).put(1).put(1).put(1);
			nwVerts.flip();
			VertexBufferObjectF nwBuf = new VertexBufferObjectF(EN.gl, nwVerts, BufferUsage.STATIC_DRAW);
			nwRect = new Mesh(DrawCommand.drawArrays(PrimitiveType.TRIANGLE_STRIP, 0, 4), 
					new AttribArray(nwBuf, VertexAttribute.POSITION, 2, DataType.FLOAT, 4*2, 0),
					new AttribArray(nwBuf, VertexAttribute.TEXTURE_COORDINATE_0, 2, DataType.FLOAT, 4*2, 0));


			if("true".equals(System.getProperty("gl.nofbo")))
				fbm = new FramebufferManagerDummy(nwRect);
			else
				fbm = new FramebufferManagerImpl(nwRect, gameConfig.depthBuffer);
			
			try {
				Fonts.loadDefaultFont(gl);
			} catch(IOException e) {
				throw new ResourceException("Failed to load default font", e);
			}
			GMLib.init();
			act.start(gl, null);
			animator.add(window);
			animator.invokeOnAnimator(() -> {
				RectangleImmutable vp = screen.getMainMonitor(window.getBounds()).getViewportInWindowUnits();
				window.setPosition(vp.getX()+(vp.getWidth()-window.getWidth())/2, vp.getY()+(vp.getHeight()-window.getHeight())/2);
			});
			status = Status.RUNNING;
		}
		
		animator.resetFPSCounter();
	}

	@Override
	public void dispose(GLAutoDrawable drawable)
	{
		if(gl != null)
		{
			if(windowDestroyed && status != Status.ENDED)
			{
				act.end(gl, ActivityChangeType.GAME_END);
				status = Status.ENDED;
			}
			animator.stop();
			GLResource.uninitAll(gl);
			gl = null;
		}
	}

	@Override
	public void display(GLAutoDrawable drawable)
	{
		try {
	    	in = ih.swap();
			if(in.getSurfaceWidth() <= 0 || in.getSurfaceHeight() <= 0)
				return;
			act.run(gl);
		} catch(ActivityChangeException ex) {
			handleActivityChange(ex, 16);
		} catch(RuntimeException err) {
			animator.stop();
			err.printStackTrace();
		}
		time++;
	}
	
	public void clear()
	{
		int clearColor = gameConfig.colorOutsideRoom;
		gl.glClearColor(
				color_get_red(clearColor)/255f, 
				color_get_green(clearColor)/255f, 
				color_get_blue(clearColor)/255f, 
				1.0f
				);
		gl.glClear(GL2ES1.GL_COLOR_BUFFER_BIT);
	}

	protected void handleActivityChange(ActivityChangeException ex, int limit)
	{
		try {
			act.end(gl, ex.type);
			if(ex.type == ActivityChangeType.GAME_END || ex.type == ActivityChangeType.GAME_RESTART)
			{
				resetGlobals();
				Room.clearPersistence();
			}
			
			if(ex.next == null) {
				status = Status.ENDED;
				close();
				return;
			} else {
				act = ex.next;
				act.start(gl, ex.type);
		    	in = ih.swap();
				act.run(gl);
			}
		} catch(ActivityChangeException e) {
			if(limit == 0)
				throw new RuntimeException("Too many instantaneous activity changes (act="+act+")");
			else
				handleActivityChange(e, limit-1);
		}
	}
	

	@Override
	public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {}

	public void close()
	{
		windowDestroyed = true;
		animator.invokeOnAnimator(() -> window.destroy());
	}
	
	public int getSurfaceWidth()
	{
		return in.getSurfaceWidth();
	}
	public int getSurfaceHeight()
	{
		return in.getSurfaceHeight();
	}
	
	public float getDPI()
	{
		float[] tmp = new float[2];
		window.getPixelsPerMM(tmp);
		return (tmp[0]+tmp[1])*25.4f/2.0f;
	}

	public double getFPS()
	{
		return animator.getLastFPS();
	}
	public void resetFrameTime()
	{
		animator.resetFrameTime();
	}
	public void setTargetFPS(int fps)
	{
		animator.setTargetFPS(fps);
	}
	public int getTargetFPS()
	{
		return animator.getTargetFPS();
	}

	public void setTitle(String caption)
	{
		animator.invokeOnAnimator(() -> window.setTitle(caption));
	}

	public boolean isFocused()
	{
		return in.isFocused();
	}
	
	public boolean isTouchMode()
	{
		return false;
	}

	public void setKeepAlive(boolean b)
	{
		//Mobile feature
	}

	public String toString(VK key)
	{
		return ih.toString(key);
	}
	
	protected void resetGlobals()
	{
		try {
			Object blank = gameConfig.global.getClass().newInstance();
			Reflect.copy(gameConfig.global, blank);
		} catch(Exception err) {
			throw new RuntimeException("Failed to reset globals", err);
		}
	}
	
	public void setVSync(boolean b)
	{
		this.vSync = b;
		gl.setSwapInterval(b?1:0);
	}

	public boolean getVSync()
	{
		return vSync;
	}
}
