package owg.lgm.room;

import com.jogamp.opengl.GL2ES1;

import owg.lgm.gl.Tex2D;
import owg.lgm.util.Letterbox;
import owg.util.compass.Compass;

import static owg.lgm.plaf.Engine.EN;

public class DirectionalTransition implements TransitionFunc
{
	protected float[] view = new float[4];
	protected int surfaceWidth, surfaceHeight;
	
	protected final Compass direction;
	protected final int mvOld;
	protected final boolean shiftOld;
	protected final int mvNew;
	protected final boolean shiftNew;

	public DirectionalTransition(Compass direction, int mvOld, boolean shiftOld, int mvNew, boolean shiftNew) {
		this.direction = direction;
		this.mvOld = mvOld;
		this.shiftOld = shiftOld;
		this.mvNew = mvNew;
		this.shiftNew = shiftNew;
	}

	@Override
	public void render(Tex2D prev, Tex2D next, float interp)
	{
		EN.gl.glMatrixMode(GL2ES1.GL_PROJECTION);
		EN.gl.glLoadIdentity();
		surfaceWidth = EN.getSurfaceWidth();
		surfaceHeight = EN.getSurfaceHeight();
		EN.gl.glOrtho(0, surfaceWidth, surfaceHeight, 0, -1, 1);
		EN.gl.glMatrixMode(GL2ES1.GL_MODELVIEW);

		EN.gl.glDisable(GL2ES1.GL_BLEND);
		renderPart(prev, mvOld, shiftOld, interp, false);
		renderPart(next, mvNew, shiftNew, interp, true);
		EN.gl.glEnable(GL2ES1.GL_BLEND);
	}

	protected void renderPart(Tex2D tex, int move, boolean shift, float interp, boolean invert)
	{
		float tx0, ty0, tx1, ty1;
		float x0, y0, x1, y1;
		
		float invInterp;
		if(invert) {
			invInterp = interp;
			interp = 1.0f-interp;
		} else
			invInterp = 1.0f-interp;
		int inversion = invert ? -1 : 1;
		
		tex.begin();
		tx0 = 0;
		ty0 = 0;
		tx1 = tex.mx;
		ty1 = tex.my;
		x0 = (move*interp*direction.sdx*inversion);
		y0 = (move*interp*direction.sdy*inversion);
		x1 = x0+1;
		y1 = y0+1;
		if(shift)
		{
			if(direction.sdx*inversion < 0)
			{
				x0 = 0;
				tx1 = tex.mx*invInterp;
			}
			if(direction.sdy*inversion < 0)
			{
				y0 = 0;
				ty1 = tex.my*invInterp;
			}
			if(direction.sdx*inversion > 0)
			{
				x1 = 1;
				tx0 = tex.mx*interp;
			}
			if(direction.sdy*inversion > 0)
			{
				y1 = 1;
				ty0 = tex.my*interp;
			}
		}
		

		Letterbox.getViewport(view, 0, surfaceWidth, surfaceHeight, tex.dataWidth, tex.dataHeight);
		EN.gl.glLoadIdentity();
		EN.gl.glTranslatef(view[0] + x0*view[2], view[1] + y0*view[3], 0);
		EN.gl.glScalef((x1-x0)*view[2], (y1-y0)*view[3], 0);
		EN.gl.glMatrixMode(GL2ES1.GL_TEXTURE);
		EN.gl.glTranslatef(tx0, ty0, 0);
		EN.gl.glScalef(tx1-tx0, ty1-ty0, 0);
		EN.gl.glColor4f(1, 1, 1, 1);
		EN.nwRect.render(EN.gl);
		EN.gl.glLoadIdentity();
		EN.gl.glMatrixMode(GL2ES1.GL_MODELVIEW);
		tex.end();
	}
}