package owg.lgm.room;

import static owg.lgm.plaf.Engine.EN;

import com.jogamp.opengl.GL2ES1;

import owg.lgm.gl.BlendMode;
import owg.lgm.gl.Tex2D;
import owg.lgm.util.Letterbox;

public class FadeTransition implements TransitionFunc
{
	protected float[] views = new float[8];
	protected int surfaceWidth, surfaceHeight;
	
	protected final boolean fadeToBlack;
	
	public FadeTransition(boolean fadeToBlack)
	{
		this.fadeToBlack = fadeToBlack;
	}
	
	@Override
	public void render(Tex2D prev, Tex2D next, float interp)
	{
		EN.gl.glMatrixMode(GL2ES1.GL_PROJECTION);
		EN.gl.glLoadIdentity();
		surfaceWidth = EN.getSurfaceWidth();
		surfaceHeight = EN.getSurfaceHeight();
		EN.gl.glOrtho(0, surfaceWidth, surfaceHeight, 0, -1, 1);
		Letterbox.getViewport(views, 0, surfaceWidth, surfaceHeight, prev.dataWidth, prev.dataHeight);
		Letterbox.getViewport(views, 4, surfaceWidth, surfaceHeight, next.dataWidth, next.dataHeight);
		
		float aa, ab;
		if(fadeToBlack) {
			aa = 1-2*interp;
			ab = 2*interp-1;
		}
		else
		{
			aa = 1-interp;
			ab = interp;
		}
		BlendMode.ADD.set();
		if(aa>0)
			renderPart(prev, views, 0, aa);
		if(ab > 0)
			renderPart(next, views, 4, ab);
		BlendMode.NORMAL.set();
	}

	protected void renderPart(Tex2D tex, float[] baseViewport, int baseViewportOff, float a)
	{
		float w = baseViewport[baseViewportOff+2];
		float h = baseViewport[baseViewportOff+3];
		float x0 = baseViewport[baseViewportOff  ];
		float y0 = baseViewport[baseViewportOff+1];
		
		tex.begin();
		EN.gl.glMatrixMode(GL2ES1.GL_MODELVIEW);
		EN.gl.glLoadIdentity();
		EN.gl.glTranslatef(x0, y0, 0);
		EN.gl.glScalef(w, h, 0);
		
		EN.gl.glMatrixMode(GL2ES1.GL_TEXTURE);
		EN.gl.glScalef(tex.mx, tex.my, 1);
		EN.gl.glColor4f(1, 1, 1, a);
		EN.nwRect.render(EN.gl);
		EN.gl.glLoadIdentity();
		EN.gl.glMatrixMode(GL2ES1.GL_MODELVIEW);
		tex.end();
	}
}
