package owg.lgm.util;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import owg.util.rethrow.Rethrow;

public class Reflect
{
	public static<S, T extends S> T copy(T target, S src)
	{
		try {
			//Copy public instance fields
			for(Field sf : src.getClass().getFields()) {
				if(!Modifier.isStatic(sf.getModifiers()))
					sf.set(target, sf.get(src));
			}
			//Copy get/set method pairs
			for(Method sm : src.getClass().getMethods())
			{
				if(sm.getName().startsWith("get") && sm.getName().length()>3 && Character.isUpperCase(sm.getName().charAt(3)) && sm.getParameterCount() == 0) {
					try {
						Method tm = target.getClass().getMethod("set"+sm.getName().substring(3), sm.getReturnType());
						tm.invoke(target, sm.invoke(src));
					} catch(NoSuchMethodException e) {
						//This is fine
					}
				}
			}
			return target;
		} catch(IllegalAccessException | InvocationTargetException err) {
			throw Rethrow.rethrow(err);
		}
	}
	
	public static<S, T extends S> T[] arrayCopy(T[] target, S[] src)
	{
		try {
			for(int i = 0; i<target.length; i++) {
				@SuppressWarnings("unchecked")
				T t = (T) target.getClass().getComponentType().newInstance();
				target[i] = t;
				copy(target[i], src[i]);
			}
			return target;
		} catch(InstantiationException | IllegalAccessException err) {
			throw Rethrow.rethrow(err);
		}
	}
}
